/** @jsx jsx */

import { Global, jsx } from '@emotion/core';
import { Fragment, ReactNode, memo } from 'react';

import { useFocusModality } from '@elemental-ui-alpha/a11y'; // hmm, a11y can't depend on core now. damn circular ref...
import {
  ElementalNamedTheme,
  ElementalProvider,
  useElementalTheme,
} from '@elemental-ui-alpha/theme';

import { normalize } from './normalize';

type CoreProps = {
  /** The app content. */
  children: ReactNode;
  /** Include styles to normalize element styles among browsers. */
  includeNormalize?: boolean;
  /** Optimize text rendering with CSS. */
  optimizeLegibility?: boolean;
  /** The current theme object. */
  theme?: ElementalNamedTheme;
};

// TODO: handle fontFace imports e.g.
// @font-face {
// 	font-family: CustomFont;
// 	font-weight: normal;
// 	src: url("${fontFace.normal}")
// }
// @font-face {
// 	font-family: CustomFont;
// 	font-weight: bold;
// 	src: url("${fontFace.bold}")
// }

export const Core = ({
  children,
  includeNormalize = true,
  optimizeLegibility = true,
  theme,
}: CoreProps) => {
  return (
    <ElementalProvider theme={theme}>
      <RootStyles
        includeNormalize={includeNormalize}
        optimizeLegibility={optimizeLegibility}
      />
      <App>{children}</App>
    </ElementalProvider>
  );
};

// NOTE: only re-render leaf nodes that listen to theme changes

const App = memo(props => <Fragment {...props} />);

// Root Styles
// ------------------------------

type RootStyleProps = Omit<CoreProps, 'children' | 'theme'>;

const RootStyles = ({
  includeNormalize,
  optimizeLegibility,
}: RootStyleProps) => {
  const theme = useElementalTheme();
  return (
    <Fragment>
      <FocusRing color={theme.foreground.focusRing} />
      {includeNormalize && <Global styles={normalize} />}
      <Global
        styles={{
          html: {
            fontSize: 'initial !important', // ensure user's font-size settings are observed, for rems
          },

          body: {
            backgroundColor: theme.background.base,
            color: theme.foreground.text,
            fontSize: '1rem',
            fontWeight: 400,
            lineHeight: 1.428571429,
            fontFamily:
              '-apple-system, BlinkMacSystemFont, "Segoe UI", Roboto, Helvetica, Arial, sans-serif, "Apple Color Emoji", "Segoe UI Emoji", "Segoe UI Symbol"',

            // optimize legibility
            ...(optimizeLegibility && {
              fontFeatureSettings: '"liga" 1', // Enable OpenType ligatures in IE
              textRendering: 'optimizeLegibility',
              WebkitFontSmoothing: 'antialiased',
              MozOsxFontSmoothing: 'grayscale',
              MozFontFeatureSettings: '"liga" on',
            }),
          },

          // TODO: universal "selection" styles
          // TODO: universal "focus-ring" for "tabbable" elements
        }}
      />
    </Fragment>
  );
};

// Experimental Focus Ring
// ------------------------------

// TODO: @mitchelhamilton review perf with @jossmac

/* 
	Tabbable Element Selector
	This selector will select elements that can be focused by the tab key according to the HTML 4.01 and HTML5 specs.
	HTML5: http://www.whatwg.org/specs/web-apps/current-work/multipage/editing.html#focus
	HTML 4.01: http://www.w3.org/TR/html401/interact/forms.html#h-17.11
*/

const tabbableElements = `a[href]:focus, area:focus, button:enabled:focus, input:not([type='hidden']):enabled:focus, link[href]:focus, object:focus, select:enabled:focus, textarea:enabled:focus, [tabindex]:focus, [draggable]:focus`;

// NOTE: actual styles should come from theme

const FocusRing = ({ color }: { color: string }) => {
  const modality = useFocusModality();
  const styles =
    modality === 'keyboard'
      ? { outline: `3px solid ${color}`, outlineOffset: 2 }
      : { outline: 0 };

  return <Global styles={{ [tabbableElements]: styles }} />;
};
